unit DtCal97;

(******************************************************************************
TCalendar97

Derived from
    tCustomPanel

Properties
  Not published
    DayArray       : array which contains all days names
    MonthArray     : array which contains all monthes names
    ValidDates     : List of valid dates (read only)

  Published
    BevelInner,
    BevelOuter,
    BorderStyle,
    Enabled,
    Font,
    Hint,
    ParentFont,
    ParentShowHint,
    PopupMenu,
    ShowHint,
    TabStop,
    TabOrder       : inherited from tCustomPanel
    Colors         : Colors for the popup calendar (title, arrows, dates, ...)
      ColorArrow   : Color for arrow buttons of calendar
      ColorDates   : Color for dates of calendar
      ColorTitle   : Color for title of calendar
      ColorValid   : Color for valid dates of calendar
      ColorWeekEnd : Color for week-end dates of calendar
    DayNames       : Names of each days
    KeyRepeat      : Delay when a key is repeated
    MonthNames     : Names of each monthes
    OnlyMonth      : If set to true, can select month without days.
    OnlyValidDates : If set to true, only valid dates are allowed. Others are grayed
    StartOfWeek    : First day of week
    StartOfWeekEnd : First day of week-end: saturday, or thursday for musulmans
    ShowCancel     : Show (or hide) cancel button
    ShowNumWeek    : Show (or hide) week number column
    ShowToday      : Show (or hide) today button
    WeekEndDays    : Number of days in a week-end: 1 or 2, or more if you want!

Event
  OnDblClick,
  OnMouseUp        : inherited from tCustomPanel
  AfterNewMonth    : After changing month
  BeforeNewMonth   : Before changing month
  OnDateChange     : When date is changing
  OnDateClick      : When mouse is clicked on a date
  OnKeyCancel      : When cancel key is pressed (esc key)
  OnKeyEnter       : When enter key is pressed
  OnKillFocus      : When Calendar loose focus (mgs WM_KILLFOCUS)

Author name=BOURMAD Mehdi
Author E-mail=bourmad@multimania.com
Author URL=www.multimania.com/bourmad
******************************************************************************)

interface

uses
  dialogs,

  WinTypes, Messages, WinProcs, Classes, Controls, DsgnIntf, ExtCtrls, Forms,
  Graphics, Menus, StdCtrls, SysUtils, DtMisc97, DtFunc97, BmBmp;

type

  TCalendar97 = class(TCustomPanel)
  private
    { Private declarations}
    FAbout: TAboutMeProperty;
    FBmpToday: tBitmap;
    FColors: TCalendarColors;
    FDate : TDateTime;
    FDayArray: TDayArray;
    FDayName: TDayName;
    FKeyRepeat: Word;
    FMonthArray: TMonthArray;
    FMonthName: TMonthName;
    FMonthPopup: tPopupMenu;
    FOnlyMonth: Boolean;
    FOnlyValidDates: Boolean;
    FRepeatBtn: tCalendarButtons;
    FStartOfWeek,
    FStartOfWeekEnd: TDayOfWeek;
    FShowCancel,
    FShowNumWeek,
    FShowToday: Boolean;
    FTimer:TTimer;
    FValidDates: tStringList;
    FBeforeNewMonth : TNewMonthEvent; {TNotifyEvent}
    FAfterNewMonth : TNewMonthEvent;  {TNotifyEvent}
    FOnDateChange,
    FOnDateClick,
    FOnKeyCancel,
    FOnKeyEnter,
    FOnKillFocus: TNotifyEvent;
    FontWidth: Integer;
    FontHeight: Integer;
    FWeekendDays : byte;  
    ButtonWidth,
    ButtonHeight,
    Border,
    TextIndent: integer;
    CurrentDay,
    CurrentYear,
    CurrentMonth,
    PreviousDay,
    PreviousYear,
    PreviousMonth: Word;
    DateArray: array[1..42] of string[2];
    CurrentDateIndex,
    PreviousDateIndex,
    PreviousDateHighlight: Integer;
    MouseIsDown: Boolean;
    NumWeekWidth,
    IdealWidth,
    IdealHeight: Word;
  protected
    { Protected declarations}
    procedure CreateWnd; override;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMSetFont(var Message: TWMSetFont); message WM_SETFONT;
    procedure SetDate (value : tDateTime);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Paint; override;
    procedure CalendarColorsChange(ACalendarColors : TObject);
    procedure DayNameChange(ADayName : TObject);
    procedure MonthNameChange(AMonthName : TObject);
    function GetDayArray: PDayArray;
    function GetMonthArray: PMonthArray;
    procedure AddPopupItems;
    procedure RefreshPopupItems;
    procedure MonthPopupClick (Sender : tObject);
    procedure SetKeyRepeat (value: word);
    procedure SetOnlyMonth (value: boolean);
    procedure SetOnlyValidDates (value: boolean);
    procedure SetShowCancel (value: boolean);
    procedure SetShowNumWeek (value: boolean);
    procedure SetShowToday (value: boolean);
    procedure SetStartOfWeek (value: TDayOfWeek);
    procedure SetStartOfWeekEnd (value: TDayOfWeek);
    procedure SetWeekEndDays (value: byte);
    procedure SetColors (value: tCalendarColors);
    function GetMonthBegin: Integer;
    function PointInRect(const rectTest: TRect; X, Y: Integer): Boolean;
    function GetIndexFromPoint(Left: Integer; Top: Integer): Integer;
    function GetMonthHeaderRect: TRect;
    function GetLeftButtonRect: TRect;
    function GetRightButtonRect: TRect;
    function GetDblLeftButtonRect: TRect;
    function GetDblRightButtonRect: TRect;
    function GetCancelButtonRect: TRect;
    function GetTodayButtonRect: TRect;
    function GetCalendarRect: TRect;
    function GetRectFromIndex(Index: Integer): TRect;
    procedure LoadDateArray;
    procedure SetDay(Days: Integer);
    function GetPrevDay : Integer;
    function GetNextDay : Integer;
    function GetPrevWeek : Integer;
    function GetNextWeek : Integer;
    function GetPrevMonth : Integer;
    function GetNextMonth : Integer;
    function GetPrevYear : Integer;
    function GetNextYear : Integer;
    Function GetGoodColor (aDate: tDateTime): tColor;
    procedure CalTimer(Sender: TObject);
    procedure DrawButtons;
    procedure DrawButtonsNavigate;
    procedure DrawButtonsBottom;
    procedure DrawButtonCancel;
    procedure EraseButtonCancel;
    procedure DrawButtonToday;
    procedure EraseButtonToday;
    procedure DrawCalendarBorder;
    procedure DrawDates;
    procedure DrawDaysHeader;
    procedure DrawMonthHeader;
    procedure EraseDateFrame(Index: Integer);
    procedure DrawFocusFrame(Index: Integer; IsUp: boolean);
    function GetNumWeek(Date: TDateTime): Integer;
  public
    { Public declarations}
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    procedure CalRefresh (Level : Integer);
    procedure SetIdealSize;
    procedure AddValidDate(aDate: TDateTime);
    procedure ClearValidDates;
    function DateInList(aDate: TDateTime): Boolean;
    {properties}
    property DayArray: PDayArray read GetDayArray;
    property MonthArray: PMonthArray read GetMonthArray;
    property ShowCancel: boolean read FShowCancel write SetShowCancel default false;
    property ValidDates: tStringList read FValidDates;
  published
    { Published declarations}
    property BevelInner;
    property BevelOuter;
    property BorderStyle;
    property Enabled;
    property Font;
    property Hint;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabStop;
    property TabOrder;
    property About: TAboutMeProperty read FAbout write FAbout;
    property Colors: TCalendarColors read FColors write SetColors;
    property Date: TDateTime read FDate write SetDate;
    property DayNames: TDayName read FDayName write FDayName;
    property KeyRepeat: Word read FKeyRepeat write SetKeyRepeat default 100;
    property MonthNames: TMonthName read FMonthName write FMonthName;
    property OnlyMonth: boolean read FOnlyMonth write SetOnlyMonth default false;
    property OnlyValidDates: boolean read FOnlyValidDates write SetOnlyValidDates default false;
    property ShowNumWeek: boolean read FShowNumWeek write SetShowNumWeek default false;
    property ShowToday: boolean read FShowToday write SetShowToday default true;
    property StartOfWeek: TDayOfWeek read FStartOfWeek write SetStartOfWeek default 1;
    property StartOfWeekEnd: TDayOfWeek read FStartOfWeekEnd write SetStartOfWeekEnd default 6;
    property WeekEndDays: byte read FWeekEndDays write SetWeekEndDays default 2;
    { Some Events}
    property OnDblClick;
    property OnMouseUp;
    property AfterNewMonth: TNewMonthEvent read FAfterNewMonth write FAfterNewMonth;
    property BeforeNewMonth: TNewMonthEvent read FBeforeNewMonth write FBeforeNewMonth;
    property OnDateChange: TNotifyEvent read FOnDateChange write FOnDateChange;
    property OnDateClick: TNotifyEvent read FOnDateClick write FOnDateClick;
    property OnKeyCancel: TNotifyEvent read FOnKeyCancel write FOnKeyCancel;
    property OnKeyEnter: TNotifyEvent read FOnKeyEnter write FOnKeyEnter;
    property OnKillFocus: TNotifyEvent read FOnKillFocus write FOnKillFocus;
  end;

implementation

{$IFNDEF WIN32}
  {$R *.R16}
{$ELSE}
  {$R *.R32}
{$ENDIF}

{-------------------------------------------------------------}
{------------------------ TCalendar97 ------------------------}
{-------------------------------------------------------------}
Constructor TCalendar97.Create(AOwner:TComponent);
begin
  inherited Create (aOwner);
{  Parent := tWinControl (Owner);}
  ControlStyle := ControlStyle - [csSetCaption];

  FBmpToday := TBitmap.Create;
  DeleteObject (FBmpToday.Handle);
  FBmpToday.ReleaseHandle;
  FBmpToday.Handle := LoadBitmap(hInstance, 'TODAY');

  FDayName := TDayName.Create;
  FDayName.OnChange := DayNameChange;

  FMonthName := TMonthName.Create;
  FMonthName.OnChange := MonthNameChange;

  FMonthPopup := TPopupMenu.Create(Self);
  AddPopupItems;

  FColors := TCalendarColors.Create;
  FColors.OnChange := CalendarColorsChange;

  FValidDates := TStringList.Create;
  FOnlyMonth := false;
  FOnlyValidDates := false;
  FStartOfWeek := 1;
  FStartOfWeekEnd := 6;
  FWeekEndDays := 2;
  FShowCancel := false;
  FShowNumWeek := false;
  FShowToday := true;
  MouseIsDown := False;

  FKeyRepeat := 100;

  FTimer := TTimer.Create(Self);
  With FTimer do
  Begin
    Enabled := False;
    Interval := FKeyRepeat;
    OnTimer := CalTimer;
  End;
  FRepeatBtn := cbtNone;
end;

Destructor TCalendar97.Destroy;
begin
  FBmpToday.Free;
  FColors.Free;
  FDayName.Free;
  FMonthName.Free;
  FMonthPopup.Free;
  FTimer.Free;
  ClearValidDates;
  FValidDates.Free;
  inherited Destroy;
end;

procedure TCalendar97.CreateWnd;
begin
  inherited CreateWnd;
  {Define initial date}
  if Date=0
    then SetDate(Now);
  {Initialize form Height & Width based on Font}
  SetIdealSize;
end;

procedure TCalendar97.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TCalendar97.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  CalRefresh (1);
  if Assigned(FOnKillFocus)
    then FOnKillFocus(Self);
end;

procedure TCalendar97.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  CalRefresh (1);
end;

procedure TCalendar97.WMSetFont(var Message: TWMSetFont);
begin
  inherited;
  SetIdealSize;
end;

procedure TCalendar97.SetDate (value : tDateTime);
begin
  if (FDate <> value)
  then begin
    if value<>0
    then begin
      DecodeDate(FDate, PreviousYear, PreviousMonth, PreviousDay);
      FDate := trunc(Value);

      {Extract date Components}
      DecodeDate(FDate, CurrentYear, CurrentMonth, CurrentDay);
      CurrentDateIndex := CurrentDay + GetMonthBegin - 1;
      PreviousDateIndex := 0;
      PreviousDateHighlight := 0;
      {Reload Date Array if month or year changed}
      if (CurrentMonth <> PreviousMonth) or (CurrentYear <> PreviousYear)
        then LoadDateArray;
    end
    else begin
      FDate := 0;
    end;
    CalRefresh (5);
  end;
end;

{******************** CalTimer ***********************}
procedure TCalendar97.CalTimer(Sender: TObject);
var
  Key : Word;
begin
  case FRepeatBtn of
    cbtPrevYear : begin
        Key := VK_Home;
        KeyDown(Key, []);
      end;
    cbtNextYear : begin
        Key := VK_End;
        KeyDown(Key, []);
      end;
    cbtPrevMonth : begin
        Key := VK_Prior;
        KeyDown(Key, []);
      end;
    cbtNextMonth : begin
        Key := VK_Next;
        KeyDown(Key, []);
      end;
  end;
end;

procedure TCalendar97.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ScreenPt : tPoint;
  Index: Integer;
  Key: Word;
begin
  ScreenPt := ClientToScreen (Point(X, Y));
  {Check if mouse was pressed in DblLeft button area}
  if PointInRect(GetDblLeftButtonRect, X, Y) then
  begin
    FRepeatBtn := cbtPrevYear;
    FTimer.Enabled := true;
    CalTimer(Self);
  end;

  {Check if mouse was pressed in DblRight button area}
  if PointInRect(GetDblRightButtonRect, X, Y) then
  begin
    FRepeatBtn := cbtNextYear;
    FTimer.Enabled := true;
    CalTimer(Self);
  end;

  {Check if mouse was pressed in Left button area}
  if PointInRect(GetLeftButtonRect, X, Y) then
  begin
    FRepeatBtn := cbtPrevMonth;
    FTimer.Enabled := true;
    CalTimer(Self);
  end;

  {Check if mouse was pressed in Right button area}
  if PointInRect(GetRightButtonRect, X, Y) then
  begin
    FRepeatBtn := cbtNextMonth;
    FTimer.Enabled := true;
    CalTimer(Self);
  end;

  {Check if mouse was pressed in Cancel button area}
  if PointInRect(GetCancelButtonRect, X, Y)
  and FShowCancel
  then begin
    Key := VK_Escape;
    KeyDown(Key, Shift);
  end;

  {Check if mouse was pressed in Today button area}
  if PointInRect(GetTodayButtonRect, X, Y)
  and FShowToday
  then begin
    Key := VK_Space;
    KeyDown(Key, Shift);
  end;

  {Check if mouse was pressed in Month header area}
  if PointInRect(GetMonthHeaderRect, X, Y)
  then begin
    RefreshPopupItems;
    FMonthPopup.Popup(ScreenPt.X, ScreenPt.Y);
  end;

  {Check if mouse was pressed in date area}
  if PointInRect(GetCalendarRect, X, Y)
  and not FOnlyMonth
  then begin
    MouseIsDown := True;
    Index := GetIndexFromPoint(X, Y);

    if (Index >= GetMonthBegin)
    and (Index < (DaysInMonth(FDate) + GetMonthBegin))
    then begin
      if (not FOnlyValidDates or DateInList (FDate + Index - CurrentDateIndex))
      then begin
        SetDay(Index - CurrentDateIndex);
        DrawFocusFrame(Index, False);
      end;
    end
    else
      MouseIsDown := False;
  end;
end;

procedure TCalendar97.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Index : Integer;
begin
   if MouseIsDown then
  begin
    MouseIsDown := False;
    if not Focused
      then SetFocus;
  end;

  {Check if mouse was pressed in date area}
  if PointInRect(GetCalendarRect, X, Y)
  then begin
    Index := GetIndexFromPoint(X, Y);
    if (Index >= GetMonthBegin) and
       (Index < (DaysInMonth(FDate) + GetMonthBegin)) and
       (not FOnlyValidDates or DateInList (FDate + Index - CurrentDateIndex))
    then begin
      if Assigned(FOnDateClick)
        then FOnDateClick(Self);
    end;
  end;

  {Check if mouse was pressed in Today button}
  if PointInRect(GetTodayButtonRect, X, Y)
  and FShowToday
  then begin
    Index := GetIndexFromPoint(X, Y);
    if (not FOnlyValidDates or DateInList (FDate + Index - CurrentDateIndex))
    then begin
      if Assigned(FOnDateClick)
        then FOnDateClick(Self);
    end;
  end;

  FTimer.Enabled := false;

  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TCalendar97.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Index: Integer;
begin
  {Check if mouse was pressed in date area}
  if PointInRect(GetCalendarRect, X, Y) then
  begin
    Index := GetIndexFromPoint(X, Y);
    if (Index >= GetMonthBegin)
    and (Index < (DaysInMonth(FDate) + GetMonthBegin))
    and (Index <> CurrentDateIndex)
    then begin
      if (not FOnlyValidDates or DateInList (FDate + Index - CurrentDateIndex))
      then begin
        if not MouseIsDown then
        begin
          if PreviousDateHighlight <> Index
          then begin
            EraseDateFrame(PreviousDateHighlight);
            DrawFocusFrame(Index, True);
            PreviousDateHighlight := Index;
          end;
        end
        else begin
          SetDay(Index - CurrentDateIndex);
          EraseDateFrame(PreviousDateHighlight);
          DrawFocusFrame(Index, False);
        end;
      end;
    end
    else
    begin
      if PreviousDateHighlight<>0
      then begin
        EraseDateFrame(PreviousDateHighlight);
        PreviousDateHighlight := 0;
      end;
    end;
  end
  else begin {mouse move not in calendar rect}
    if PreviousDateHighlight<>0
    then begin
      if PreviousDateHighlight <> CurrentDateIndex
        then EraseDateFrame(PreviousDateHighlight);
      PreviousDateHighlight := 0;
    end;
  end;
end;

procedure TCalendar97.KeyDown(var Key: Word; Shift: TShiftState);
var
  Days : Integer;
begin
  Days := 0;
  case Key of
    VK_Left: begin
      Days := GetPrevDay;
    end;

    VK_Right: begin
      Days := GetNextDay;
    end;

    VK_Up: begin
      Days := GetPrevWeek;
    end;

    VK_Down: begin
      Days := GetNextWeek;
    end;

    VK_Prior: begin
      Days := GetPrevMonth;
    end;

    VK_Next: begin
      Days := GetNextMonth;
    end;

    VK_Home: begin
      Days := GetPrevYear;
    end;

    VK_End: begin
      Days := GetNextYear;
    end;

    VK_Return: begin
      if Assigned(FOnKeyEnter)
        then FOnKeyEnter(Self);
    end;

    VK_Escape: begin
      if Assigned(FOnKeyCancel)
        then FOnKeyCancel(Self);
    end;

    VK_Space: begin
      Days := Trunc(Now) - Trunc(FDate);
    end;
  end;

  SetDay(Days);    
  PreviousDateHighlight := 0;

{  if (CurrentMonth <> PreviousMonth) or
     (CurrentYear <> PreviousYear)
    then CalRefresh (0)
    else DrawFocusFrame(CurrentDateIndex, False);}
  CalRefresh (1)
end;

procedure TCalendar97.CalRefresh (Level : Integer);
{
level 0 : refresh all calendar
level 1 : refresh only focused date
level 2 : refresh only dates
level 3 : refresh date & month header
level 4 : refresh date & days header
level 5 : refresh date, days header & month header
}
begin
  Canvas.Brush.Color := clBtnFace;
  Case Level of {it is redundant, but it works...}
    0 : begin
          {level 0 : refresh all calendar}
          DrawCalendarBorder;
          DrawMonthHeader;
          DrawDaysHeader;
          DrawDates;
          DrawButtons;
          DrawFocusFrame(CurrentDateIndex, False);
        end;
    1 : begin
          {level 1 : refresh only focused date}
          EraseDateFrame (CurrentDateIndex);
          DrawFocusFrame(CurrentDateIndex, False);
        end;
    2 : begin
          {level 2 : refresh only dates}
          DrawDates;
          DrawButtonsBottom; {because dates erase them}
          DrawFocusFrame(CurrentDateIndex, False);
        end;
    3 : begin
          {level 3 : refresh date & month header}
          DrawMonthHeader;
          DrawDates;
          DrawButtonsBottom; {because dates erase them}
          DrawFocusFrame(CurrentDateIndex, False);
        end;
    4 : begin
          {level 4 : refresh date & days header}
          DrawDaysHeader;
          DrawDates;
          DrawButtonsBottom; {because dates erase them}
          DrawFocusFrame(CurrentDateIndex, False);
        end;
    5 : begin
          {level 5 : refresh date, days header & month header}
          DrawMonthHeader;
          DrawDaysHeader;
          DrawDates;
          DrawButtonsBottom; {because dates erase them}
          DrawFocusFrame(CurrentDateIndex, False);
        end;
  end;
{  DrawMonthHeader;
  DrawDates;
  DrawFocusFrame(CurrentDateIndex, False);}
end;

procedure TCalendar97.Paint;
begin
  SetIdealSize;
  CalRefresh (0);
end;

procedure TCalendar97.CalendarColorsChange(ACalendarColors : TObject);
begin
  if Assigned(ACalendarColors)
    then CalRefresh(0);
end;

procedure TCalendar97.DayNameChange(ADayName : TObject);
begin
  if Assigned(ADayName)
    then CalRefresh(4);
end;

procedure TCalendar97.MonthNameChange(AMonthName : TObject);
begin
  if Assigned(AMonthName)
    then CalRefresh(3);
end;

function TCalendar97.GetDayArray: PDayArray;
begin
  FDayArray[1] := DayNames.Monday;
  FDayArray[2] := DayNames.Tuesday;
  FDayArray[3] := DayNames.Wednesday;
  FDayArray[4] := DayNames.Thursday;
  FDayArray[5] := DayNames.Friday;
  FDayArray[6] := DayNames.Saturday;
  FDayArray[7] := DayNames.Sunday;
  Result := @FDayArray;
end;

function TCalendar97.GetMonthArray: PMonthArray;
begin
  FMonthArray[01] := MonthNames.January;
  FMonthArray[02] := MonthNames.February;
  FMonthArray[03] := MonthNames.March;
  FMonthArray[04] := MonthNames.April;
  FMonthArray[05] := MonthNames.May;
  FMonthArray[06] := MonthNames.June;
  FMonthArray[07] := MonthNames.July;
  FMonthArray[08] := MonthNames.August;
  FMonthArray[09] := MonthNames.September;
  FMonthArray[10] := MonthNames.October;
  FMonthArray[11] := MonthNames.November;
  FMonthArray[12] := MonthNames.December;
  Result := @FMonthArray;
end;

procedure TCalendar97.AddPopupItems;
var
  aMenuItem : tMenuItem;
  i : Integer;
begin
  for i:=1 to 12 do
  begin
    aMenuItem := tMenuItem.Create (FMonthPopup);
    aMenuItem.Caption := MonthArray^[I];
    aMenuItem.OnClick := MonthPopupClick;
    FMonthPopup.Items.Add(aMenuItem);
  end;
  RefreshPopupItems;
end;

procedure TCalendar97.RefreshPopupItems;
var
  i : Integer;
begin
  For I:=1 to 12 do
  begin
    FMonthPopup.Items[i-1].Caption := MonthArray^[I];
  end;
end;

procedure TCalendar97.MonthPopupClick (Sender : tObject);
var
  NewMonth : Word;
  NewDate : tDateTime;
  Days : Integer;
begin
  NewMonth := FMonthPopup.Items.IndexOf ((Sender as tMenuItem))+1;
  Days := DaysInMonth(EncodeDate(CurrentYear, NewMonth, 1));
  if CurrentDay > Days
    then CurrentDay := Days;
  NewDate := EncodeDate (CurrentYear, NewMonth, CurrentDay);
  SetDate (NewDate);
end;

procedure TCalendar97.SetKeyRepeat (value : word);
begin
  if Value <> FKeyRepeat
  then begin
    FKeyRepeat := value;
    FTimer.Interval := FKeyRepeat;
  end;
end;

procedure TCalendar97.SetOnlyMonth (value: boolean);
begin
  if FOnlyMonth <> value
  then begin
    FOnlyMonth := value;
    SetIdealSize;
  end;
end;

procedure TCalendar97.SetOnlyValidDates (value: boolean);
begin
  if FOnlyValidDates <> value
  then begin
    FOnlyValidDates := value;
    CalRefresh (2);
  end;
end;

procedure TCalendar97.SetShowCancel (value: boolean);
begin
  if value <> FShowCancel
  then begin
    FShowCancel := value;
    if FShowCancel
      then DrawButtonCancel
      else EraseButtonCancel;
    CalRefresh (2);
  end;
end;

procedure TCalendar97.SetShowNumWeek (value: boolean);
begin
  if Value <> FShowNumWeek
  then begin
    FShowNumWeek := value;
    SetIdealSize;
    CalRefresh (0);
   end;
end;

procedure TCalendar97.SetShowToday (value: boolean);
begin
  if value <> FShowToday
  then begin
    FShowToday := value;
    if FShowToday
      then DrawButtonToday
      else EraseButtonToday;
    CalRefresh (2);
  end;
end;

procedure TCalendar97.SetStartOfWeek (value : TDayOfWeek);
begin
  if FStartOfWeek<>value
  then begin
    FStartOfWeek := value;
    LoadDateArray;
    CalRefresh (4);
  end;
end;

procedure TCalendar97.SetStartOfWeekEnd (value : TDayOfWeek);
begin
  if FStartOfWeekEnd<>value
  then begin
    FStartOfWeekEnd := value;
    CalRefresh (2);
  end;
end;

procedure TCalendar97.SetWeekEndDays (value: byte);
begin
  if FWeekEndDays <> value
  then begin
    FWeekEndDays := value;
    CalRefresh (2);
  end;
end;

procedure TCalendar97.SetColors (value: tCalendarColors);
begin
  if FColors<>value
  then begin
    FColors := value;
    CalRefresh (0);
  end;
end;

{******************* Set Ideal Size for calendar ******************}
procedure TCalendar97.SetIdealSize;
var
   tmTextMetrics: TTextMetric;
begin
  if FOnlyMonth
  then begin
    FShowNumWeek := false;
    ShowToday := false;
  end;
  {if the FontWidth is not set, determine Font Height and Width for
  positioning Dates}
  Canvas.Font := Font;
  Canvas.Pen.Color := Canvas.Font.Color;
  GetTextMetrics(Canvas.Handle, tmTextMetrics);
  FontWidth :=  Round(tmTextMetrics.tmAveCharWidth +
                      tmTextMetrics.tmAveCharWidth * 6 / 10);
  FontHeight :=  Round(tmTextMetrics.tmHeight +
                       tmTextMetrics.tmHeight / 3);

  ButtonWidth := FontWidth * 3 div 2;
  if ButtonWidth < 16
    then ButtonWidth := 16;
  ButtonHeight := ButtonWidth;
  Border := ButtonWidth div 8 + 1;
  TextIndent := Border;


  {Initialize form Height & Width based on Font}
  NumWeekWidth := FontWidth * 3;
  IdealHeight := (FontHeight * 6) + (FontHeight * 2) + (Border * 2);
  IdealWidth := ((FontWidth * 3) * 7) + (2 * Border) + (2 * TextIndent);
  if FShowNumWeek
    then IdealWidth := IdealWidth + NumWeekWidth;
  Width := IdealWidth;
  Height := IdealHeight;
  {comment: FontWidth * 3 = width for 1 character}
end;

function TCalendar97.PointInRect(const rectTest: TRect; X, Y: Integer): Boolean;
begin
  Result := ((X >= rectTest.Left) and
             (X <= rectTest.Right) and
             (Y >= rectTest.Top) and
             (Y <= rectTest.Bottom));
end;

{********************* Get Month Header Rectangle *****************
 ***** Get the rectangle used for the month header.           *****
 ******************************************************************}
function TCalendar97.GetMonthHeaderRect: TRect;
var
  TempRect: TRect;
begin
  {Define Month Header Rectangle}
  TempRect.Top := Border;
  TempRect.Bottom := FontHeight;
  TempRect.Left := Border + TextIndent + ButtonWidth*2+1;
  TempRect.Right := IdealWidth - Border - TextIndent - ButtonWidth*2-1;
{  TempRect.Right := IdealWidth - Border - TextIndent - ButtonWidth*2;}

  Result := TempRect;
end;

{********************* Get DblLeft Button Rectangle ***************
 ***** Get the rectangle used for the dblleft button.         *****
 ******************************************************************}
function TCalendar97.GetDblLeftButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define DblLeft Button Rectangle}
   TempRect.Top := Border;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Left := Border + TextIndent;
   TempRect.Right := TempRect.Left + ButtonWidth;

   Result := TempRect;
end;

{********************* Get Left Button Rectangle ******************
 ***** Get the rectangle used for the left button.            *****
 ******************************************************************}
function TCalendar97.GetLeftButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Left Button Rectangle}
   TempRect.Top := Border;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Left := Border + TextIndent + ButtonWidth;
   TempRect.Right := TempRect.Left + ButtonWidth;

   Result := TempRect;
end;

{******************** Get DblRight Button Rectangle ***************
 ***** Get the rectangle used for the dblright button.        *****
 ******************************************************************}
function TCalendar97.GetDblRightButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define DblRight Button Rectangle}
   TempRect.Top := Border;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Right := IdealWidth - Border - TextIndent;
   TempRect.Left := TempRect.Right - ButtonWidth;

   Result := TempRect;
end;

{******************** Get Right Button Rectangle ******************
 ***** Get the rectangle used for the right button.           *****
 ******************************************************************}
function TCalendar97.GetRightButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Right Button Rectangle}
   TempRect.Top := Border;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Right := IdealWidth - Border - TextIndent - ButtonWidth;
   TempRect.Left := TempRect.Right - ButtonWidth;

   Result := TempRect;
end;

{******************** Get Cancel Button Rectangle *****************
 ***** Get the rectangle used for the Cancel button.          *****
 ******************************************************************}
function TCalendar97.GetCancelButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Cancel (Bottom, Right) Button Rectangle}
   TempRect.Top := IdealHeight - ButtonWidth - Border - 1;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Right := IdealWidth - Border - TextIndent;
{   TempRect.Right := Border + TextIndent + (FontWidth * 3 * 7);}
   TempRect.Left := TempRect.Right - ButtonWidth;

   Result := TempRect;
end;

{********************* Get Today Button Rectangle *****************
 ***** Get the rectangle used for the Today button.           *****
 ******************************************************************}
function TCalendar97.GetTodayButtonRect: TRect;
var
  TempRect: TRect;
begin
   {Define Today (Bottom, Right) Button Rectangle}
   TempRect.Top := IdealHeight - ButtonWidth - Border - 1;
   TempRect.Bottom := TempRect.Top + ButtonHeight;
   TempRect.Right := IdealWidth - Border - TextIndent - ButtonWidth;
   TempRect.Left := TempRect.Right - (FBmpToday.Width+4);

   Result := TempRect;
end;

{********************** Get Calendar Rectangle ********************
 ***** Get the rectangle used for the calendar section        *****
 ******************************************************************}
function TCalendar97.GetCalendarRect: TRect;
var
  TempRect: TRect;
begin
  TempRect.Left   := Border + TextIndent;
  if FShowNumWeek
    then TempRect.Left := TempRect.Left + NumWeekWidth;
  TempRect.Top    := (FontHeight * 2) + Border;
  TempRect.Bottom := IdealHeight - Border;
  TempRect.Right  := IdealWidth - Border - TextIndent;

  Result := TempRect;
end;

{******************** Get Rectangle From Index ********************
 ***** Get the rectangle used for this day section            *****
 ******************************************************************}
function TCalendar97.GetRectFromIndex(Index: Integer): TRect;
var
  TempRect: TRect;
  Week: Integer;
  Day: Integer;
begin
  TempRect := GetCalendarRect;

  with TempRect do
  begin
    case Index of
      01..07: Week := 1;
      08..14: Week := 2;
      15..21: Week := 3;
      22..28: Week := 4;
      29..35: Week := 5;
      36..42: Week := 6;
    else {see Delphi Warnings}
      Week := 1;
    end;

    Day := Index - ((Week - 1) * 7);

    Left := Left + ((FontWidth * 3) * (Day - 1));
    Top := (FontHeight * Week) + FontHeight + Border;
    Bottom := Top +  FontHeight;
    Right := Left + fontWidth * 3;
  end;
  Result := TempRect;
end;

{*************************** Get Month Begin **************************
 ***** This function Gets the index value of the first day of the *****
 ***** month.                                                     *****
 **********************************************************************}
function TCalendar97.GetMonthBegin: Integer;
var
  DOW: Integer;
begin
  DOW := DayOfWeek(EncodeDate(CurrentYear, CurrentMonth, 1)) - 1;
  if DOW = 0 then DOW := 7;
  Result := (DOW - FStartOfWeek + 7) mod 7 + 1;
end;

{****************** Get Index From Point ******************
 **********************************************************}
function TCalendar97.GetIndexFromPoint(Left: Integer ; Top: Integer): Integer;
var
  Index, Week, Day: Integer;
  Res: Real;
  TempRect: Trect;
begin
  TempRect := GetCalendarRect;
  Index := -1;
  {Is point in the calendar rectangle?}
  if (Left > TempRect.Left)  and (Top > TempRect.Top)
  and (Left < TempRect.Right) and (Top < TempRect.Bottom) then
  begin
    {Determine the week number of the selected date}
    Res   := (Top - TempRect.Top)  / FontHeight;
    Week  := Trunc(Res);
    Res   := (Left - TempRect.Left) /(FontWidth * 3);
    Day   := Trunc(Res) + 1;

    Index := Day + (Week * 7)
  end;
  Result := Index;
end;

{********************** GetNumWeek ***********************}
function TCalendar97.GetNumWeek (Date: TDateTime): Integer;
var
  Y, M, D,           
  FDay : Word;
  JanF : TDateTime;
  Days : Integer;
begin
  try
    DecodeDate(Date, Y, M, D);
    JanF := EncodeDate(Y, 1, 1);
    FDay := DayOfWeek(JanF);
    Days := Trunc(Int(Date)-JanF)+7-DayOfWeek(Date-1);
    Inc(Days, 7 * Ord(FDay in [2..5]));
    Result := Days div 7;
    if Result = 0 then
    Begin
      if (DayOfWeek(EncodeDate(Y-1,  1,  1)) > 5)
      or (DayOfWeek(EncodeDate(Y-1, 12, 31)) < 5) then
        Result := 52
      else
        Result := 53;
    end
    else
      if Result = 53 then
        if (FDay > 5) or (DayOfWeek(EncodeDate(Y, 12, 31)) < 5) then
          Result := 1;
  except
    Result := 0;
  end;
end;

{********************** LoadDateArray *********************}
procedure TCalendar97.LoadDateArray;
var
  Index,
  BeginIndex,
  EndIndex: Integer;
  FromDate, TillDate : TDateTime;
begin
  BeginIndex := GetMonthBegin;
  EndIndex := BeginIndex + DaysInMonth(FDate) - 1;

  FromDate := EncodeDate(CurrentYear, CurrentMonth, 1);
  TillDate := EncodeDate(CurrentYear, CurrentMonth, DaysInMonth(FDate));

  if ((CurrentMonth <> PreviousMonth)
  or (CurrentYear <> PreviousYear))
  and Assigned(FBeforeNewMonth)
    then FBeforeNewMonth (Self, FromDate, TillDate);

  for Index := 1 to 42 do
  begin
    if (Index < BeginIndex) or (Index > EndIndex) then
      DateArray[Index] := '  '
    else
      DateArray[Index] := IntToStr((Index - BeginIndex) + 1);
  end;

  CalRefresh (3); {force refresh before raise AfterNewMonth event}

  if ((CurrentMonth <> PreviousMonth)
  or (CurrentYear <> PreviousYear))
  and Assigned(FAfterNewMonth)
    then FAfterNewMonth (Self, FromDate, TillDate);
end;

{***************** Set Date **************************
 **** This procedure adjusts the date by nDays    ****
 **** nDays can be possitive or negative. It      ****
 **** also populates the vars YEAR, MONTH and DAY ****
 *****************************************************}
procedure TCalendar97.SetDay(Days: Integer);
begin
  if (Days<>0)
  then begin
    {Save current date information}
    PreviousDateIndex := CurrentDateIndex;
    PreviousDateHighlight := CurrentDateIndex;
    DecodeDate(FDate, PreviousYear, PreviousMonth, PreviousDay);

    {Change the date and update member variables}
    FDate := FDate + Days;
    if FOnlyMonth
      then FDate := GetFirstOfMonth(FDate);
    DecodeDate(FDate, CurrentYear, CurrentMonth, CurrentDay);
    CurrentDateIndex := (GetMonthBegin + CurrentDay) - 1;

    {Reload Date Array if month or year changed}
    if (CurrentMonth <> PreviousMonth) or (CurrentYear <> PreviousYear)
      then LoadDateArray;

    if Assigned(FOnDateChange)
      then FOnDateChange(Self);
  end;
end;

{******************** Get Previous Day ********************}
function TCalendar97.GetPrevDay : Integer;
var
  i : integer;
begin
  if not FOnlyValidDates
  then
    Result := -1
  else begin
    i := FValidDates.IndexOf (FormatDateTime(ShortDateFormat, FDate));
    if i in [1..FValidDates.Count-1]
      then Result := Trunc((FValidDates.Objects[i-1] as tDateObject).Date)
                   - Trunc(FDate)
      else Result := 0;
  end;
end;

{********************* Get Next Day ***********************}
function TCalendar97.GetNextDay : Integer;
var
  i : integer;
begin
  if not FOnlyValidDates
  then
    Result := +1
  else begin
    i := FValidDates.IndexOf (FormatDateTime(ShortDateFormat, FDate));
    if i in [0..FValidDates.Count-2]
      then Result := Trunc((FValidDates.Objects[i+1] as tDateObject).Date)
                   - Trunc(FDate)
      else Result := 0;
  end;
end;

{******************** Get Previous Week *******************}
function TCalendar97.GetPrevWeek : Integer;
begin
  Result := -7;
end;

{******************** Get Next Week ***********************}
function TCalendar97.GetNextWeek : Integer;
begin
  Result := +7;
end;

{******************** GetPreviousMonth ********************}
function TCalendar97.GetPrevMonth : Integer;
var
  Days: Integer;
  Month: Integer;
begin
  if CurrentMonth > 1 then
    Month := CurrentMonth - 1
  else
    Month := 12;

  Days := DaysInMonth(EncodeDate(CurrentYear, Month, 1));
  if CurrentDay > Days then Days := CurrentDay;
  Result := -Days;
end;

{******************** Get Next Month **********************}
function TCalendar97.GetNextMonth : Integer;
var
  NextMonthDays : integer;
begin
  if CurrentMonth = 12
  then
    NextMonthDays := DaysInMonth(EncodeDate(CurrentYear, 1, 1))
  else
    NextMonthDays := DaysInMonth(EncodeDate(CurrentYear, CurrentMonth+1, 1));
  if CurrentDay > NextMonthDays
  then begin
    Result := DaysInMonth(FDate)-CurrentDay+NextMonthDays;
  end
  else begin
    Result := DaysInMonth(FDate);
  end;
end;

{******************* GetPrevious Year *********************}
function TCalendar97.GetPrevYear : Integer;
var
  NbDays : integer;
begin
  NbDays := 365;
  {if the current year is a leap year and the date is
   after February 29, subtract 1 day}
  if IsLeapYear(FDate) and (CurrentMonth > 3)
    then inc(NbDays);

  {if the Previous year is a leap year and the date is
   before February 29, subtract 1 day}
  if IsLeapYear(FDate) and (CurrentMonth < 3)
    then inc(NbDays);

  Result := -NbDays;
end;

{******************** Get Next Year ***********************}
function TCalendar97.GetNextYear : Integer;
var
  NbDays : integer;
begin
  NbDays := 365;
  {if the current year is a leap year and the date is
   before February 29, add 1 day}
  if IsLeapYear(FDate) and (CurrentMonth < 3)
    then inc(NbDays);

  {if the current year is a leap year and the date is
   after February 29, add 1 day}
  if IsLeapYear(FDate) and (CurrentMonth > 3)
    then inc(NbDays);

  Result := NbDays;
end;

{******************* GetGoodColor *************************}
Function TCalendar97.GetGoodColor (aDate: tDateTime): tColor;
var
  WeekEnd : Set of tDayOfWeek;
  Day2    : tDayOfWeek;
  i, DOW  : integer;
begin
  if not FOnlyMonth
  then begin
    if DateInList(aDate) then
      Result := Colors.ColorValid
    else begin
      DOW := DayOfWeek(aDate) - 1;
      if DOW = 0
        then DOW := 7;

      WeekEnd := [];
      Day2 := FStartOfWeekEnd;
      for i := 1 to FWeekEndDays do
      begin
        WeekEnd := WeekEnd + [Day2];
        if Day2 = tDayOfWeek(7)
          then Day2 := tDayOfWeek(1)
          else Day2 := succ(Day2);
      end;

      if DOW in WeekEnd
        then Result := Colors.ColorWeekEnd
        else Result := Colors.ColorDates
    end;
  end
  else begin
    Result := Colors.ColorDates;
  end;
end;

{******************** Draw Buttons ***********************}
procedure TCalendar97.DrawButtons;
begin
  {draw navigation buttons}
  DrawButtonsNavigate;

  {draw bottom buttons}
  DrawButtonsBottom;
end;

{**************** Draw Navigate Buttons *******************}
procedure TCalendar97.DrawButtonsNavigate;
var
  LeftButtonRect,
  RightButtonRect,
  DblLeftButtonRect,
  DblRightButtonRect: TRect;
  OldStyle: TBrushStyle;
  ArrowWMarg,
  ArrowHMarg: integer;
begin
  with Canvas do
  begin
    OldStyle := Brush.Style;
    LeftButtonRect := GetLeftButtonRect;
    RightButtonRect := GetRightButtonRect;
    DblLeftButtonRect := GetDblLeftButtonRect;
    DblRightButtonRect := GetDblRightButtonRect;
    {erase background}
    Brush.Color := clBtnFace;
    FillRect(LeftButtonRect);
    FillRect(RightButtonRect);
    FillRect(DblLeftButtonRect);
    FillRect(DblRightButtonRect);

    {Select Solid Pen}
    Pen.Style := psSolid;
    Pen.Width := 1;

    {Create Embossed effect - Outline left & upper in white}
    Pen.Color := clBtnHighlight;
    MoveTo(LeftButtonRect.Left, LeftButtonRect.Bottom - 1);
    LineTo(LeftButtonRect.Left, LeftButtonRect.Top);
    LineTo(LeftButtonRect.Right - 1, LeftButtonRect.Top);

    MoveTo(RightButtonRect.Left, RightButtonRect.Bottom - 1);
    LineTo(RightButtonRect.Left, RightButtonRect.Top);
    LineTo(RightButtonRect.Right - 1, RightButtonRect.Top);

    MoveTo(DblLeftButtonRect.Left, DblLeftButtonRect.Bottom - 1);
    LineTo(DblLeftButtonRect.Left, DblLeftButtonRect.Top);
    LineTo(DblLeftButtonRect.Right - 1, DblLeftButtonRect.Top);

    MoveTo(DblRightButtonRect.Left, DblRightButtonRect.Bottom - 1);
    LineTo(DblRightButtonRect.Left, DblRightButtonRect.Top);
    LineTo(DblRightButtonRect.Right - 1, DblRightButtonRect.Top);

    {Create Embossed effect - Outline right & bottom in gray}
    Pen.Color := clBtnShadow;
    MoveTo(LeftButtonRect.Right - 1, LeftButtonRect.Top);
    LineTo(LeftButtonRect.Right - 1, LeftButtonRect.Bottom - 1);
    LineTo(LeftButtonRect.Left, LeftButtonRect.Bottom - 1);

    MoveTo(RightButtonRect.Right - 1, RightButtonRect.Top);
    LineTo(RightButtonRect.Right - 1, RightButtonRect.Bottom - 1);
    LineTo(RightButtonRect.Left, RightButtonRect.Bottom - 1);

    MoveTo(DblLeftButtonRect.Right - 1, DblLeftButtonRect.Top);
    LineTo(DblLeftButtonRect.Right - 1, DblLeftButtonRect.Bottom - 1);
    LineTo(DblLeftButtonRect.Left, DblLeftButtonRect.Bottom - 1);

    MoveTo(DblRightButtonRect.Right - 1, DblRightButtonRect.Top);
    LineTo(DblRightButtonRect.Right - 1, DblRightButtonRect.Bottom - 1);
    LineTo(DblRightButtonRect.Left, DblRightButtonRect.Bottom - 1);

    ArrowWMarg := ButtonWidth div 4;
    ArrowHMarg := ButtonHeight div 4;

    {Draw Arrows}
    Brush.Color := Colors.ColorArrow;
    Brush.Style := bsSolid;
    Pen.Color := Colors.ColorArrow;
    Polygon([Point (LeftButtonRect.Right - ArrowWMarg -3,
                      LeftButtonRect.Top + ArrowHMarg),
             Point (LeftButtonRect.Right - ArrowWMarg -3,
                      LeftButtonRect.Bottom - ArrowHMarg),
             Point (LeftButtonRect.Left+ButtonWidth div 2 -3,
                      LeftButtonRect.Top + ButtonHeight div 2)]);

    Polygon([Point (RightButtonRect.Left+ButtonWidth div 2 -1,
                      RightButtonRect.Top + ArrowHMarg),
             Point (RightButtonRect.Left+ButtonWidth div 2 -1,
                      RightButtonRect.Bottom - ArrowHMarg),
             Point (RightButtonRect.Right - ArrowWMarg -1,
                      RightButtonRect.Top + ButtonHeight div 2)]);

    {Draw Double Arrows}
    Brush.Color := Colors.ColorArrow;
    Brush.Style := bsSolid;
    Pen.Color := Colors.ColorArrow;
    Polygon([Point (DblLeftButtonRect.Right - ArrowWMarg,
                      DblLeftButtonRect.Top + ArrowHMarg),
             Point (DblLeftButtonRect.Right - ArrowWMarg,
                      DblLeftButtonRect.Bottom - ArrowHMarg),
             Point (DblLeftButtonRect.Left+ButtonWidth div 2,
                      DblLeftButtonRect.Top + ButtonHeight div 2)]);

    Polygon([Point (DblLeftButtonRect.Left + ButtonWidth div 2 - 1,
                      DblLeftButtonRect.Top + ArrowHMarg),
             Point (DblLeftButtonRect.Left + ButtonWidth div 2 - 1,
                      DblLeftButtonRect.Bottom - ArrowHMarg),
             Point (DblLeftButtonRect.Left + ArrowWMarg - 1,
                      DblLeftButtonRect.Top + ButtonHeight div 2)]);

    Polygon([Point (DblRightButtonRect.Left+ButtonWidth div 2,
                      DblRightButtonRect.Top + ArrowHMarg),
             Point (DblRightButtonRect.Left+ButtonWidth div 2,
                      DblRightButtonRect.Bottom - ArrowHMarg),
             Point (DblRightButtonRect.Right - ArrowWMarg,
                      DblRightButtonRect.Top + ButtonHeight div 2)]);

    Polygon([Point (DblRightButtonRect.Left + ArrowWMarg - 1,
                      DblRightButtonRect.Top + ArrowHMarg),
             Point (DblRightButtonRect.Left + ArrowWMarg - 1,
                      DblRightButtonRect.Bottom - ArrowHMarg),
             Point (DblRightButtonRect.Left + ButtonWidth div 2-1,
                      DblRightButtonRect.Top + ButtonHeight div 2)]);

    Brush.Style := OldStyle;
    Brush.Color := clBtnFace;
    Pen.Color := clBlack;
    Pen.Width := 1;
  end;
end;

{******************* Draw Bottom Buttons **********************}
procedure TCalendar97.DrawButtonsBottom;
begin
  {draw Cancel button}
  if FShowCancel
    then DrawButtonCancel
    else EraseButtonCancel;

  {draw Today button}
  if FShowToday
    then DrawButtonToday
    else EraseButtonToday;
end;

{******************** Draw Button Cancel ***********************}
procedure TCalendar97.DrawButtonCancel;
var
  CancelButtonRect: TRect;
begin
  CancelButtonRect := GetCancelButtonRect;

  with Canvas do
  begin
    {erase background}
    Brush.Color := clBtnFace;
    FillRect(CancelButtonRect);

    {Select Solid Pen}
    Pen.Style := psSolid;

    {Create Embossed effect - Outline left & upper in white}
    Pen.Color := clBtnHighlight;
    Pen.Width := 1;
    MoveTo(CancelButtonRect.Left, CancelButtonRect.Bottom - 1);
    LineTo(CancelButtonRect.Left, CancelButtonRect.Top);
    LineTo(CancelButtonRect.Right - 1, CancelButtonRect.Top);

    {Create Embossed effect - Outline right & bottom in gray}
    Pen.Color := clBtnShadow;
    Pen.Width := 1;
    MoveTo(CancelButtonRect.Right - 1, CancelButtonRect.Top);
    LineTo(CancelButtonRect.Right - 1, CancelButtonRect.Bottom - 1);
    LineTo(CancelButtonRect.Left, CancelButtonRect.Bottom - 1);

    Pen.Color := clMaroon;
    Pen.Width := 2;
    MoveTo(CancelButtonRect.Left + 5, CancelButtonRect.Bottom - 4);
    LineTo(CancelButtonRect.Right - 5, CancelButtonRect.Top + 4);
    Pen.Width := 2;
    MoveTo(CancelButtonRect.Left + 5, CancelButtonRect.Top + 4);
    LineTo(CancelButtonRect.Right - 5, CancelButtonRect.Bottom - 4);
    Pen.Width := 1;
  end;
end;

{******************** Erase Button Cancel ********************}
procedure TCalendar97.EraseButtonCancel;
var
  CancelButtonRect: TRect;
begin
  CancelButtonRect := GetCancelButtonRect;
  Canvas.Brush.Color := clBtnFace;
  Canvas.FillRect(CancelButtonRect);
end;

{******************** Draw Button Today ***********************}
procedure TCalendar97.DrawButtonToday;
var
  BmpRect,
  TodayButtonRect: TRect;
begin
  TodayButtonRect := GetTodayButtonRect;

  with Canvas do
  begin
    {erase background}
    Brush.Color := clBtnFace;
    FillRect(TodayButtonRect);

    {Select Solid Pen}
    Pen.Style := psSolid;

    {Create Embossed effect - Outline left & upper in white}
    Pen.Color := clBtnHighlight;
    Pen.Width := 1;
    MoveTo(TodayButtonRect.Left, TodayButtonRect.Bottom - 1);
    LineTo(TodayButtonRect.Left, TodayButtonRect.Top);
    LineTo(TodayButtonRect.Right - 1, TodayButtonRect.Top);

    {Create Embossed effect - Outline right & bottom in gray}
    Pen.Color := clBtnShadow;
    Pen.Width := 1;
    MoveTo(TodayButtonRect.Right - 1, TodayButtonRect.Top);
    LineTo(TodayButtonRect.Right - 1, TodayButtonRect.Bottom - 1);
    LineTo(TodayButtonRect.Left, TodayButtonRect.Bottom - 1);

  end;
  BmpRect := TodayButtonRect;
  InflateRect (BmpRect, -2, -2);
  DrawTransparentBitmap (Canvas, FBmpToday, BmpRect,
                         FBmpToday.Canvas.Pixels[0,0],
                         false);
end;

{******************** Erase Button Today ********************}
procedure TCalendar97.EraseButtonToday;
var
  TodayButtonRect: TRect;
begin
  TodayButtonRect := GetTodayButtonRect;
  Canvas.Brush.Color := clBtnFace;
  Canvas.FillRect(TodayButtonRect);
end;

{*************** Draw Calendar Border *********************}
procedure TCalendar97.DrawCalendarBorder;
var
  MargeW, MargeH : Integer;
  rectDraw: TRect;
begin
  rectDraw := Rect (0, 0, IdealWidth, IdealHeight);
  with Canvas do
  begin
    {Select Black Pen to outline Window}
    Pen.Style := psSolid;
    Pen.Width := 1;
    Pen.Color := clBlack;
    Brush.Color := clBtnFace; 
    Brush.Style := bsSolid;

    {BorderStyle}
    Case BorderStyle of
      bsSingle : begin
                   MargeW := 2;
                   MargeH := 2;
                 end;
      bsNone   : begin
                   MargeW := 0;
                   MargeH := 0;
                 end;
      else ;
    end;
    {BevelInner}
    Case BevelInner of
      bvRaised  : {Create Embossed effect - Outline left & upper in white}
                  Pen.Color := clBtnHighlight;
      bvLowered : {Create Embossed effect - Outline left & upper in gray}
                  Pen.Color := clBtnShadow;
      bvNone    : {Create not effect}
                  Pen.Color := clBtnFace;
    end;
    MoveTo(1, rectDraw.Bottom - MargeH - 2);
    LineTo(1, 1);
    LineTo(rectDraw.Right - MargeW - 2, 1);

    Case BevelInner of
      bvRaised :  {Create Embossed effect - Outline right & bottom in gray}
                  Pen.Color := clBtnShadow;
      bvLowered : {Create Embossed effect - Outline right & bottom in white}
                  Pen.Color := clBtnHighlight;
      bvNone    : {Create not effect}
                  Pen.Color := clBtnFace;
    end;
    LineTo(rectDraw.Right - MargeW - 2, rectDraw.Bottom - MargeH - 2);
    LineTo(1, rectDraw.Bottom - MargeH - 2);

    {BevelOuter}
    Case BevelOuter of
      bvRaised  : {Create Embossed effect - Outline left & upper in white}
                  Pen.Color := clBtnHighlight;
      bvLowered : {Create Embossed effect - Outline left & upper in gray}
                  Pen.Color := clBtnShadow;
      bvNone    : {Create not effect}
                  Pen.Color := clBtnFace;
    end;
    MoveTo(0, rectDraw.Bottom - MargeH - 1);
    LineTo(0, 0);
    LineTo(rectDraw.Right - MargeW - 1, 0);

    Case BevelOuter of
      bvRaised :  {Create Embossed effect - Outline right & bottom in gray}
                  Pen.Color := clBtnShadow;
      bvLowered : {Create Embossed effect - Outline right & bottom in white}
                  Pen.Color := clBtnHighlight;
      bvNone    : {Create not effect}
                  Pen.Color := clBtnFace;
    end;
    LineTo(rectDraw.Right - MargeW - 1, rectDraw.Bottom - MargeH - 1);
    LineTo(0, rectDraw.Bottom - MargeH - 1);
  end;
end;

{*********************** Draw Dates ***********************}
procedure TCalendar97.DrawDates;
var
  Index,
  Week,
  Day: Integer;
  NumWeekStr : String;
  NumWeek,
  pDate: PChar;
  CalRect,
  TempRect: Trect;
  aDate: TDateTime;
  NumWeekDsp: Boolean;
begin
  pDate := StrAlloc(3);
  NumWeek := StrAlloc (3);

  CalRect := GetCalendarRect;
  {erase background}
  Canvas.Brush.Color := clBtnFace;
  Canvas.FillRect(CalRect);

  {Define normal font}
  Canvas.Font := Font;
  {Cycle through the weeks}
  for Week := 1 to 6 do
  begin
    TempRect.Top    := CalRect.Top + (Week - 1) * FontHeight;
    TempRect.Bottom := TempRect.Top + FontHeight;
    NumWeekDsp := False;
    {Cycle through the days}
    for Day := 1 to 7 Do
    begin
      Index := Day + ((Week - 1) * 7);

      StrPCopy(pDate, DateArray[Index]);
      if DateArray[Index] <> '  ' then
      begin
        aDate := EncodeDate (CurrentYear, CurrentMonth,
                              StrToInt(DateArray[Index]));
        if FShowNumWeek and not NumWeekDsp
        then begin
          TempRect.Left  := Border;
          TempRect.Right := TempRect.Left + (FontWidth * 3);
          {erase background}
          Canvas.Brush.Color := clBtnFace;
          Canvas.FillRect(TempRect);
          Canvas.Font.Color := Colors.ColorTitle;
          NumWeekStr := IntToStr(GetNumWeek(aDate));
          StrPCopy (NumWeek, NumWeekStr);
          DrawText(Canvas.Handle, NumWeek, Length(NumWeekStr), TempRect, DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE);
          NumWeekDsp := True;
        end;
        Canvas.Font.Color := GetGoodColor (aDate);
      end
      else
        aDate := 0;

      TempRect.Left  := CalRect.Left + ((FontWidth * 3) * (Day - 1));
      TempRect.Right := TempRect.Left + (FontWidth * 3);

      if ((FOnlyValidDates) and not DateInList(aDate))
      or not enabled
      then begin {draw disabled dates}
        DrawDisabledText (Canvas, pDate, Length(DateArray[Index]), TempRect,
                  DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE);
      end
      else begin
        DrawText (Canvas.Handle, pDate, Length(DateArray[Index]), TempRect,
                  DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE);
      end;
    end;
  end;
  StrDispose(pDate);
  StrDispose(NumWeek);
end;

{*********************** Draw Day Header ************************}
procedure TCalendar97.DrawDaysHeader;
var
  DayNo,
  I: Integer;
  pDay: PChar;
  TempRect: Trect;
begin
  pDay := StrAlloc(3);

  {Calculate Rect Top.  2nd line = FontHeight * 2}
  TempRect := ClientRect;
  TempRect.Top := FontHeight + Border;
  TempRect.Bottom := TempRect.Top + FontHeight;
  TempRect.Left := TempRect.Left + Border;
  TempRect.Right := TempRect.Right - Border;

  {erase background}
  Canvas.Brush.Color := clBtnFace;
  Canvas.FillRect(TempRect);

  {display week number header}
  if FShowNumWeek
  then begin
    TempRect.Right := TempRect.Left + (FontWidth * 3);
    DrawText(Canvas.Handle, 'N', 2, TempRect,(DT_CENTER or DT_TOP or DT_SINGLELINE));
    TempRect.Right := TempRect.Right + TextIndent;
  end
  else
    TempRect.Right := TempRect.Left + TextIndent;

  {Calculate each date rect.
   rect = FontWidth * 3 (width of two chars + space)}
  {Cycle through the days}
  for I := 1 to 7 do
  begin
    DayNo := (FStartOfWeek + I - 2) mod 7 + 1;
    StrPCopy(pDay, DayArray^[DayNo]);
    TempRect.Left := TempRect.Right;
    TempRect.Right := TempRect.Right + FontWidth * 3;
    DrawText(Canvas.Handle, pDay, 2, TempRect, (DT_CENTER or DT_TOP or DT_SINGLELINE));
  end;

  {Define normal font}
  Canvas.Font := Font;   
  {Draw line below days}
  TempRect.Bottom := IdealHeight - 4;
  TempRect.Left := ClientRect.Left + Border;
  TempRect.Right := IdealWidth - Border - TextIndent;

  Canvas.Pen.Color := clBtnShadow;
  Canvas.MoveTo(TempRect.Left , TempRect.Top + FontHeight - 3);
  Canvas.LineTo(TempRect.Right, TempRect.Top + FontHeight - 3);
  if FShowNumWeek
  then begin
    Canvas.MoveTo(TempRect.Left + NumWeekWidth, TempRect.Top + 2);
    Canvas.LineTo(TempRect.Left + NumWeekWidth, TempRect.Bottom);
  end;
  Canvas.Pen.Color := clWhite;
  Canvas.MoveTo(TempRect.Left,  TempRect.Top + FontHeight - 2);
  Canvas.LineTo(TempRect.Right, TempRect.Top + FontHeight - 2);
  if FShowNumWeek
  then begin
    Canvas.MoveTo(TempRect.Left + NumWeekWidth + 1, TempRect.Top + 2);
    Canvas.LineTo(TempRect.Left + NumWeekWidth + 1, TempRect.Bottom);
  end;

  StrDispose(pDay);
end;

{******************** Draw Month Header *******************}
procedure TCalendar97.DrawMonthHeader;
var
   Month: String;
   pMonth: PChar;
   TempRect: Trect;
begin
  {Define normal font}
  Canvas.Font := Font;
 {Draw in Bold font}
  Canvas.Font.Style := Canvas.Font.Style + [fsBold];
  Canvas.Font.Color := Colors.ColorTitle;
  Month := MonthArray^[CurrentMonth] + ' ' + IntToStr(CurrentYear);

  pMonth := StrAlloc(Length(Month) + 1);
  StrPCopy(pMonth, Month);

  TempRect := GetMonthHeaderRect;

  Canvas.Brush.Color := clBtnFace;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(TempRect);

  DrawText(Canvas.Handle, pMonth, Length(Month), TempRect,
          (DT_CENTER or DT_VCENTER or DT_BOTTOM or DT_SINGLELINE));
  StrDispose(pMonth);
end;

{******************** Erase Date Frame ********************}
procedure TCalendar97.EraseDateFrame(Index: Integer);
var
  pDate:PChar;
  TempRect: TRect;
  aDate: TDateTime;
begin
  if Index > 0
  then begin
    if DateArray[Index] <> '  '
    then begin
      aDate := EncodeDate(CurrentYear, CurrentMonth,
                           StrToInt(DateArray[Index]));
    end
    else
      aDate := 0;
    pDate := StrAlloc(3);
    {Define normal font}
    Canvas.Font := Font;

    StrPCopy(pDate, DateArray[Index]);
    Canvas.Font.Color := GetGoodColor(aDate);
    Canvas.Brush.Color := clBtnFace;
    TempRect := GetRectFromIndex(Index);
    Canvas.FillRect(TempRect);
    if not FOnlyValidDates or DateInList (aDate)
    then begin
      DrawText(Canvas.Handle, pDate, Length(DateArray[Index]),
               TempRect, (DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE));
    end
    else begin
      DrawDisabledText(Canvas, pDate, Length(DateArray[Index]),
               TempRect, (DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE));
    end;
    StrDispose(pDate);
  end;
end;

{******************** Draw Focus Frame ********************}
procedure TCalendar97.DrawFocusFrame(Index: Integer; IsUp: boolean);
{if not Up, then Down ... for button style}
var
  pDate:PChar;
  TempRect: TRect;
  aDate: TDateTime;
  CoulShadow,
  CoulHighlight: tColor;
begin
  if not FOnlyMonth and (Index > 0) and (Index < 42)
  and enabled
  then begin
    if (DateArray[Index] <> '  ')
    then begin
      aDate := EncodeDate(CurrentYear, CurrentMonth, StrToInt(DateArray[Index]));
      pDate := StrAlloc(3);
      {Erase Previous Date Focus}
      EraseDateFrame(PreviousDateIndex);

      {Define normal font}
      Canvas.Font := Font;
      {Draw the Date in Bold font}
      Canvas.Font.Style := Canvas.Font.Style + [fsBold];
      Canvas.Font.Color := GetGoodColor (aDate);

      TempRect := GetRectFromIndex(Index);
      if not FOnlyValidDates or DateInList (aDate)
      then begin
        StrPCopy(pDate, DateArray[Index]);
        DrawText(Canvas.Handle, pDate, Length(DateArray[Index]),
                 TempRect, (DT_CENTER or DT_VCENTER or DT_TOP or DT_SINGLELINE));
      end;

      if IsUp
      then begin
        CoulShadow := clWhite;
        CoulHighlight := clBtnShadow;
      end
      else begin
        if FDate<>0
        then begin
          CoulShadow := clBtnShadow;
          CoulHighlight := clWhite;
        end
        else begin
          CoulShadow := clBtnFace;
          CoulHighlight := clBtnFace;
        end;
      end;

      {Frame date with Shadow}
      Canvas.Pen.Color := CoulShadow;
      Canvas.MoveTo(TempRect.Left, TempRect.Bottom - 1);
      Canvas.LineTo(TempRect.Left, TempRect.Top);
      Canvas.LineTo(TempRect.Right - 1, TempRect.Top);

      {Frame date with Highlight}
      Canvas.Pen.Color := CoulHighlight;
      Canvas.LineTo(TempRect.Right - 1, TempRect.Bottom - 1);
      Canvas.LineTo(TempRect.Left, TempRect.Bottom - 1);

      if Focused and TabStop and not IsUp
      then begin
        InflateRect (TempRect, -2, -2);
        Canvas.DrawFocusRect(TempRect);
      end;

      StrDispose(pDate);
    end;
  end;
end;

(********* Add a Date in the list of valid dates **********)
procedure TCalendar97.AddValidDate(aDate: TDateTime);
var
  DateObj: tDateObject;
  i : Integer;
begin
  DateObj := tDateObject.Create;
  DateObj.Date := aDate;
  i := 0;
  while (i <= FValidDates.Count-1)
  and ((FValidDates.Objects[i] as tDateObject).Date<=aDate) do
    inc (i);
  if i > FValidDates.Count
    then i := FValidDates.Count;
  FValidDates.InsertObject(i, FormatDateTime(ShortDateFormat, aDate), DateObj);
  CalRefresh(2);  
end;

(************** Clear list of valid dates ***************)
procedure TCalendar97.ClearValidDates;
begin
  if not Assigned (FValidDates)
    then Exit;
  while FValidDates.Count > 0 do
  begin
    FValidDates.Delete(0);
  end;
end;

(********* Is the supplied data in the date list? *********)
function TCalendar97.DateInList(aDate: TDateTime): Boolean;
var
  DateObj: tDateObject;
  i: Integer;
begin
  Result := False;
  if FValidDates.Count>0
  then begin
    for i := 0 to FValidDates.Count - 1 do
    begin
      DateObj := tDateObject(FValidDates.Objects[i]);
      if (DateObj.Date = aDate) then
      begin
        Result := True;
        Break;
      end;
    end;
  end;
end;

end.
